"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto = require('crypto').webcrypto;
/**
 * class for express request handling
 */
class Handlers {
    /**
     *
     * @param options HandlersOptions
     */
    constructor(options) {
        this.tokenGenerator = options.tokenGenerator;
        this.salt = options.salt;
    }
    /**
     *
     * @param req Request
     * @param res Response
     */
    async generateComponentToken(req, res) {
        /** expected inputs
         * {
         *   "domain": "string",
         *   "room": "string",
         *   "tenant": "string",
         *   "tokenType": "JIGASI"
         * }
         **/
        const inputs = {
            ...req.params,
            ...req.body
        };
        req.context.logger.debug('inputs', { inputs,
            body: req.body,
            params: req.params });
        const payload = {
            room: inputs.room ? inputs.room : '*'
        };
        const signOptions = {
            issuer: 'jaas-components',
            expiresIn: '1 day'
        };
        if (inputs.tenant) {
            signOptions.subject = inputs.tenant;
        }
        else if (inputs.domain) {
            signOptions.subject = inputs.domain;
        }
        else {
            signOptions.subject = '*';
        }
        switch (inputs.tokenType) {
            case 'JIGASI':
                // inject any jigasi specifics here
                signOptions.audience = `jigasi.${inputs.domain}`;
                break;
        }
        const token = this.tokenGenerator.clientToken(req.context, payload, signOptions);
        res.status(200).json({ token });
    }
    /**
     *
     * @param req Request
     * @param res Response
     */
    async generateServerToken(req, res) {
        let signOptions = {
            audience: 'jitsi-token-generator',
            expiresIn: '1 hour'
        };
        if (req.params) {
            signOptions = {
                ...signOptions,
                ...req.params
            };
        }
        if (req.body) {
            signOptions = {
                ...signOptions,
                ...req.body
            };
        }
        const token = this.tokenGenerator.serverToken(req.context, signOptions);
        res.status(200).json({ token });
    }
    /**
     * Generates a client token valid for 2 hours. The token structure is taken from the body of the reques.
     * @param req Request
     * @param res Response
     * @returns A json with token and tenant (extracted from the token body field sub).
     */
    async generateClientToken(req, res) {
        const inputs = {
            ...req.body,
            ...req.params,
            ...req.query
        };
        req.context.logger.debug('inputs', { inputs,
            method: req.method,
            body: req.body,
            params: req.params });
        const payload = {
            ...req.body
        };
        const signOptions = {
            audience: 'jitsi',
            issuer: 'chat',
            expiresIn: '2 hour'
        };
        const token = this.tokenGenerator.clientToken(req.context, payload, signOptions);
        let e2eeKey;
        if (inputs.e2eeKey === 'true') {
            const keyMaterial = await crypto.subtle.importKey('raw', payload.room, 'PBKDF2', false, ['deriveKey']);
            const dk = await crypto.subtle.deriveKey({
                name: 'PBKDF2',
                salt: this.salt,
                iterations: 100000,
                hash: 'SHA-256'
            }, keyMaterial, {
                name: 'AES-GCM',
                length: 128
            }, true, ['encrypt', 'decrypt']);
            const toHexString = (bytes) => Array.from(new Uint8Array(bytes), (byte) => 
            // eslint-disable-next-line no-bitwise
            `0${(byte & 0xff).toString(16)}`.slice(-2)).join('');
            e2eeKey = toHexString(await crypto.subtle.exportKey('raw', dk));
        }
        const tenant = inputs.sub;
        res.status(200).json({
            token,
            tenant,
            e2eeKey
        });
    }
}
exports.default = Handlers;
